--[[ $Id: Core.lua 78023 2008-07-07 21:57:15Z ag $ ]]--
local LibStub = LibStub
local Fortress = LibStub("AceAddon-3.0"):NewAddon("Fortress", "AceConsole-3.0", "AceEvent-3.0")

local broker = LibStub("LibDataBroker-1.1")
local legos  = LibStub("LegoBlock-Beta1")

--local L = LibStub("AceLocale-3.0"):GetLocale("Fortress")

local function debug(...)
--	local tmp = {"Fortress: ", ...}
--	ChatFrame1:AddMessage(table.concat(tmp, " "))
end

local db
local labelColor, unitColor
local GameTooltip = GameTooltip

local function Block_OnEnter(self)
	local obj = self.obj
	
	if obj.tooltipText and (not db.hideTTInCombat or not InCombatLockdown()) then
		GameTooltip:ClearAllPoints()
		GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
		GameTooltip:SetText(obj.tooltipText)
		self.hideTooltipOnLeave = true
	end
	if obj.OnEnter then
		obj.OnEnter(self)
	end
end

local function Block_OnLeave(self)
	local obj = self.obj
	if self.hideTooltipOnLeave then
		GameTooltip:Hide()
		self.hideTooltipOnLeave = nil
	end
	if obj.OnLeave then
		obj.OnLeave(self)
	end
end

local frames = {}
local dataObjects = {}
local strTable = {}
local updaters = {
	text = function(frame, value, name)
		local object = dataObjects[name]
		if value then	
			local str
			if object.info and object.label then
				if type(object.info) == "string" and type(object.unit) == "string" then
					str = string.format("%s|cff%s%s|r",object.info, unitColor, object.unit)
				elseif type(object.info) == "table" and type(object.unit) == "table" then
					for k,v in ipairs(object.info) do
						strTable[k] = string.format("%s|cff%s%s|r", v, unitColor, object.unit[k])
					end
					str = table.concat(strTable, " ")
				end
			else
				str = value
			end
			if db.pluginDB[name].showLabel and type(object.label) == "string" then
				str = string.format("|cff%s%s:|r %s", labelColor, object.label, str)
			end			
			frame:SetText(str)
			frame:ShowText()
		else
			frame:HideText()
		end
	end,
	icon = function(frame, value)
		if value then
			frame:SetIcon(value)
			frame.db.hasIcon = true
		end
		if frame.db.showIcon then
			frame:ShowIcon()
		else
			frame:HideIcon()
		end
	end,
	OnClick = function(frame, value)
		frame:SetScript("OnClick", value)
	end,
	OnEnter = function(frame, value)
		frame:SetScript("OnEnter", value)
	end,
	OnLeave = function(frame, value)
		frame:SetScript("OnLeave", value)
	end,
}

function Fortress:OnInitialize()
	local defaults = {
		profile = {
			blockDB = {
				['*'] = {},
			},
			pluginDB = {
				['*'] = {
					enabled = true,
					showIcon = true,
					hideInCombat = false,
					borderColor = {r = 0, g = 0, b = 0, a = 0.7},
					frameColor = {r = 0, g = 0, b = 0, a = 0.3},
					fontSize = 12,
					showLabel = true,
				},
			},
			enabled = true,
			hideInCombat = false,
			hideTTInCombat = false,
			labelColor = { r = 1, g = 1, b = 1 },
			unitColor = { r = 1, g = 1, b = 1 },
		},
	}
	self.db = LibStub("AceDB-3.0"):New("FortressDB", defaults)
	db = self.db.profile
	
	self:SetEnabledState(db.enabled)
	self:RegisterOptions()
	
	local colorDb = Fortress.db.profile.unitColor
	local r, g, b = colorDb.r, colorDb.g, colorDb.b
	unitColor = ("%02x%02x%02x"):format(r*255, g*255, b*255)	
	
	colorDb = Fortress.db.profile.unitColor
	r, g, b = colorDb.r, colorDb.g, colorDb.b
	labelColor = ("%02x%02x%02x"):format(r*255, g*255, b*255)	
end

function Fortress:OnEnable()
	for name, obj in broker:DataObjectIterator() do
		self:LibDataBroker_DataObjectCreated(nil, name, obj)
	end
	broker.RegisterCallback(self, "LibDataBroker_DataObjectCreated")
	
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
end

function Fortress:OnDisable()
	broker.UnregisterAllCallbacks(self)
	for _, f in pairs(frames) do
		f:Hide()
	end
end

function Fortress:LibDataBroker_DataObjectCreated(event, name, obj)
	debug("Dataobject Registered:", name)
	if db.pluginDB[name].enabled then
		self:EnableDataObject(name, obj)
	end
	self:AddObjectOptions(name)
end

function Fortress:AttributeChanged(event, name, key, value)
	if not db.pluginDB[name].enabled then return end
	local f = frames[name]
	local obj = dataObjects[name]
	
	local update = updaters[key]
	if update then
		update(f, value, name)
	end
end

function Fortress:UpdateObject(name, obj)
	local frame = frames[name]
	obj = obj or broker:GetDataObjectByName(name)
	for key, func in pairs(updaters) do
		if obj[key] then 
			func(frame, obj[key], name) 
		end
	end	
	self:UpdateColor(name)
	self:UpdateFont(name)	
end

function Fortress:EnableDataObject(name, obj)
	db.pluginDB[name].enabled = true
	dataObjects[name] = obj
	-- create frame for object
	local frame = frames[name] or legos:New("Fortress"..name, nil, nil, db.blockDB[name])
	frames[name] = frame
	frame.db = db.pluginDB[name]
	frame.obj = obj
	
	frame:SetScript("OnEnter", Block_OnEnter)
	frame:SetScript("OnLeave", Block_OnLeave)
	frame:RegisterForClicks("LeftButtonUp","RightButtonUp")
	
	-- update display
	self:UpdateObject(name, obj)
	self:UpdateColor(name)
	self:UpdateFont(name)
	frame:Show()
	
	-- register update callback
	broker.RegisterCallback(self, "LibDataBroker_AttributeChanged_"..name, "AttributeChanged")
end

function Fortress:DisableDataObject(name, obj)
	db.pluginDB[name].enabled = false
	broker.UnregisterCallback(self, "LibDataBroker_AttributeChanged_"..name)
	if frames[name] then
		frames[name]:Hide()
	end
end

function Fortress:PLAYER_REGEN_DISABLED()
	local force = db.hideInCombat
	for _, f in pairs(frames) do
		if force or f.db.hideInCombat then
			f:Hide()
		end
	end
end

function Fortress:PLAYER_REGEN_ENABLED()
	for _, f in pairs(frames) do
		if f.db.enabled then
			f:Show()
		end
	end
end

function Fortress:UpdateColor(name)
	local frame = frames[name]
	local borderColor = db.pluginDB[name].borderColor
	local frameColor = db.pluginDB[name].frameColor
	frame:SetBackdropBorderColor(borderColor.r,borderColor.g,borderColor.b,borderColor.a)
	frame:SetBackdropColor(frameColor.r,frameColor.g,frameColor.b,frameColor.a)
end

function Fortress:UpdateFont(name)
	local frame = frames[name]
	local fontName = frame.text:GetFont()
	local fontSize = db.pluginDB[name].fontSize
	frame.text:SetFont(fontName, fontSize)
end